int START_TIME = 8, //Start of the day; 24-hour time
END_TIME = 19,  //The end time of the day; 24-hour time
WEEK_DAYS = 5, //The number of days in the week
CLOCK_ROLL = 12,  //12 or 24. Whether to use military time. 

PADDING = 5,        //The width of lines used to split days
LEFT_MARGIN = 80,   //The margin on the left for times. 
RIGHT_MARGIN = 10,  //Whitespace on right. 
TOP_MARGIN  = 30,   //The margin on top for days. 
BOTTOM_MARGIN = 10, //Whitespace on bottom

DPI = 92, //Dots per inch that you plan to print at
WIDTH = 7, //Width and height, inches, of final product. 
HEIGHT = 10; //(If too large, the window will go offscreen.

PFont HEADINGS = createFont("SansSerif.plain", 16);
PFont MAIN_TEXT = createFont("SansSerif.plain", 12);

ArrayList<Class> classes = new ArrayList<Class>();
String selected = null; //The active file.
void setup() {
  size(DPI*WIDTH, DPI*HEIGHT);
  textFont(HEADINGS);
  frameRate(1);
  selectInput("Select a schedule-formatted file:", "selectFile");
}

void draw() {
  readFile();
  //Draw redraws the calendar background. Also blanks the visible schedule.
  background(255);

  //Hour lines
  stroke(255/2);
  strokeWeight(PADDING/2);
  for (int i = START_TIME; i < END_TIME; i++) {
    line(LEFT_MARGIN*.7, time2y(i), day2x(WEEK_DAYS), time2y(i));
  }
  
  //Half hour lines
  stroke(255*3/4);
  strokeWeight(1);
  for (int i = START_TIME; i <= END_TIME; i++) {
    line(LEFT_MARGIN, time2y(i+.5), day2x(WEEK_DAYS), time2y(i+.5));
  }
  
  //Times
  fill(0);
  textAlign(RIGHT, CENTER);
  for (int i = START_TIME; i < END_TIME; i++) {
    text(time2string(i), LEFT_MARGIN*.6, time2y(i));
  }
  
  //Fill in the classes.
  for (Class c : classes) c.draw();
  
  //Day separation lines
  stroke(0);
  strokeWeight(PADDING);
  for (int i = 0; i <= WEEK_DAYS; i++) {
    line(day2x(i), TOP_MARGIN/2, day2x(i), time2y(END_TIME));
  }
  
  //Bottom line to 'seal' calendar
  line(LEFT_MARGIN, time2y(END_TIME), day2x(WEEK_DAYS), time2y(END_TIME));
  
  //Day names
  String[] names;
  if (WEEK_DAYS == 5) names = new String[] {"Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "", ""};
  else if (WEEK_DAYS == 7) names = new String[] {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
  else names = new String[] {""};
  fill(0);
  textAlign(CENTER, BOTTOM);
  for (int i = 0; i < names.length; i++) {
    text(names[i], day2x(i) + dayWidth()/2, TOP_MARGIN);
  }
}  

//Converts a day (Sunday is 0, Monday is 1... into an X-position for a box origin).
int day2x(int day) {
  return LEFT_MARGIN + day*dayWidth();
}

//Returns the width of each day, in pixels. Hardcoded for now; will be friendlier later.
int dayWidth() {
  return (width-LEFT_MARGIN-RIGHT_MARGIN)/WEEK_DAYS;
}

int time2y(float time) {
  return int((time-START_TIME) / (END_TIME - START_TIME) * (height - TOP_MARGIN-BOTTOM_MARGIN) + TOP_MARGIN);
}

//Parses a string; returns a 24-hour float time in fractional hours. 
float string2time(String time) {
  //Find markers in the time: am/pm, : 
  int addTime = 0;
  int indexOfA = time.toLowerCase().indexOf("a");
  int indexOfP = time.toLowerCase().indexOf("p");
  int indexOfC = time.indexOf(":");
  
  //Remove AM/PM; set a marker for +12 hours if pm
  if (indexOfA != -1) time = time.substring(0, indexOfA);
  if (indexOfP != -1) {
    time = time.substring(0, indexOfP);
    addTime = 12;
  }
  
  //Split time around the colon for h:mm
  //Whitespace is trimmed - float() borks on spaces.
  if (indexOfC != -1) {
    int minutes = int(time.substring(indexOfC+1).trim());
    return float(time.substring(0, indexOfC).trim()) + minutes/60.0 + addTime;
  }
  //If there's no colon, treat the time as hours. 
  return float(time.trim())+addTime;
}

//Parses an integer hour and returns a time in am/pm or 24 hour, depending on CLOCK_ROLL
String time2string(int time) {
  if (CLOCK_ROLL == 24) {
    return str(time);
  }
  
  if (CLOCK_ROLL == 12 && time > 12) {
    return str(time-12)+" PM";
  } else if (CLOCK_ROLL == 12 && time == 12) {
    return "12 PM"; //The 12-hour clock is weird.
  } else if (CLOCK_ROLL == 12 && time < 12) {
    return str(time) + " AM";
  }
  return "CLOCK_ROLL incorrect.";
}

void keyPressed() {
  if (key == 'o')   selectInput("Select a schedule-formatted file:", "selectFile");
  if (key == 's')  {
    selectOutput("Where shall we save the file?", "screenshot");
  }
}
